<?php
/*
Plugin Name: limit_ja_login
Plugin URI: 
Description: 管理画面へのログインを日本からのIPアドレスのみに制限をかけるプラグインです。
Author: 武蔵浦和なび
Version: 1.0
Author URI: https://musashiurawa.navi-local.com/
*/
 
class limitjalogin{

	public $limitjalogin_db_version = '1.0';

	function __construct(){
		add_action('admin_menu', array($this, 'limitjalogin_add_pages'));
		register_activation_hook( __FILE__, array($this,'limitjalogin_install'));
		register_uninstall_hook( __FILE__, array($this,'limitjalogin_delete') );
		
		add_action( 'admin_init', array($this, 'limitjalogin_submit') );
		add_action( 'login_form', array($this, 'limitjalogin_submit') );
		add_action( 'admin_enqueue_scripts', array($this, 'limitjalogin_styles') );
	}
	
	function limitjalogin_add_pages() {
		add_options_page(
			'設定',
			'ログイン制限',
			'administrator',
			'limitjalogin',
			array($this,'limitjalogin_admin')
		);
	}

	function limitjalogin_styles() {
		$plugin_url = plugin_dir_url( __FILE__ );
		wp_enqueue_style('limitjalogin_style', $plugin_url . 'css/limit_ja_login.css' );
	}
	
	function limitjalogin_admin() {

		global $wpdb;

		//オブション画面に表示する内容
		if ( isset($_POST['limitlogin_mailaddress']) && isset($_POST['limitlogin_logdatecount'])) {
			if(is_numeric($_POST['limitlogin_logdatecount']) && $_POST['limitlogin_logdatecount'] > 0 ){
				
				check_admin_referer('limitlogin_accesslimit');
				$opt1 = $_POST['limitlogin_mailaddress'];
				$opt2 = $_POST['limitlogin_logdatecount'];
				update_option('limitlogin_mailaddress', $opt1);
				update_option('limitlogin_logdatecount', $opt2);
				?><div class="updated fade"><p><strong><?php _e('Options saved.'); ?></strong></p></div><?php
			}
		}

		//ログを取得（直近50件）
		$table_name = $wpdb->prefix.'limitlogin_log';
		$crows = $wpdb->get_results("SELECT * FROM $table_name ORDER BY id desc limit 50 ");
		?>
		<div class="tabs">
			<input id="all" type="radio" name="tab_item" checked>
			<label class="tab_item" for="all">設定</label>
			<input id="log" type="radio" name="tab_item">
			<label class="tab_item" for="log">ログ（直近50件）</label>
			<div class="tab_content" id="all_content">
				<div class="tab_content_description">
					<div class="wrap">
						<div id="icon-options-general" class="icon32"><br /></div>
						<h2>ログイン制限設定</h2>
						<form action="" method="post">
							<?php
							wp_nonce_field('limitlogin_accesslimit');
							$opt1 = get_option('limitlogin_mailaddress');
							$mailaddress = isset($opt1) && $opt1!='' ? $opt1: "";
							$opt2 = get_option('limitlogin_logdatecount');
							$limit_count = isset($opt2) && $opt2!='' ? $opt2: 90;
							?> 
							<table class="form-table">
								<tr valign="top">
									<th scope="row"><label for="inputtext">通知先メールアドレス</label>
									</th>
									<td>
										<input name="limitlogin_mailaddress" type="text" value="<?php  echo $mailaddress ?>" class="large-text" />
										<br>※不正アクセスを検知した場合に通知されます
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><label for="inputtext">ログ保存期限</label></th>
									<td><input name="limitlogin_logdatecount" type="text" value="<?php  echo $limit_count ?>" class="small-text" />
									</td>
								</tr>
							</table>
							
							<p class="submit"><input type="submit" name="Submit" class="button-primary" value="変更を保存" /></p>
						</form>
					</div>
				</div>
			</div>
			<div class="tab_content" id="log_content">
				<div class="tab_content_description">
					<div class="wrap">
						<table>
							<tr>
								<th>時刻</th>
								<th>国</th>
								<th>IPアドレス</th>
								<th>リファラー</th>
								<th>判定</th>
							</tr>
							<?php foreach($crows as $row){ ?>
							<tr>
								<td style="white-space: nowrap;"><?php echo date("Y/m/d H:i:s",strtotime($row->time)); ?></td>
								<td><?php echo $row->country; ?></td>
								<td><?php echo $row->ipaddress; ?></td>
								<td><?php echo htmlspecialchars($row->reffer,ENT_QUOTES,'UTF-8') ?></td>
								<?php 
									if($row->flg == 1) {
										echo '<td class="color_ok">OK</td>';
									}else{
										echo '<td class="color_ng">NG</td>';
									}
								?>
							</tr>
							<?php } ?>
						</table>
					</div>
				</div>
			</div>
		</div>
		
		<?php
	}
	
	function limitjalogin_install(){
		//初期化処理
		global $wpdb;
		
		$table_name = $wpdb->prefix . 'limitlogin_log';
		$charset_collate = $wpdb->get_charset_collate();
		
		$sql = "CREATE TABLE $table_name (
			id mediumint(9) NOT NULL AUTO_INCREMENT,
			time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			ipaddress varchar(50) NOT NULL,
			country varchar(50) NOT NULL,
			reffer text NOT NULL,
			info text NOT NULL,
			flg SMALLINT NOT NULL,
			UNIQUE KEY id (id)
		  ) $charset_collate;";
		
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );
		
		add_option( 'limitjalogin_db_version', $this->limitjalogin_db_version );
	}
	function limitjalogin_delete(){
		global $wpdb;
		$table_name = $wpdb->prefix . 'limitlogin_log';
		$sql = "DROP TABLE IF EXISTS {$table_name}";
		$wpdb->query($sql);

		delete_option('limitlogin_mailaddress');
		delete_option('limitlogin_logdatecount');
		delete_option('limitjalogin_db_version');
	}
	function limitjalogin_submit(){
	
		global $wpdb;
		
		$ip = $_SERVER["REMOTE_ADDR"];
		$refferer = $_SERVER["HTTP_USER_AGENT"];
		
		//IPアドレスから国をwhoisを用いて取得
		$output = array();
		exec("whois ".$ip,$output);
		
		$country = null;
		foreach($output as $value){
			if(strpos($value,"country:") === 0){
				$country = str_replace(array("country:"," ","　"),"",$value);
			}
		}
		if($country == null || $country == ""){
			$country = "NON";
		}
		
		//国がJPかどうか判定
		$isOK = true;
		if($country == 'JP'){
			$isOK = true;
		}else{
			$isOK = false;
		}
		
		$flg = "";
		if($isOK){
			$flg = "1";
		}else{
			$flg = "9";
		}
		
		//アクセスレコードを登録
		if (!is_user_logged_in()) {
			//ログインしていない場合のみ
			$table_name = $wpdb->prefix.'limitlogin_log';
			$wpdb->insert(
				$table_name,
				array(
				  'time' => current_time( 'mysql' ),
				  'ipaddress' => $ip,
				  'country' => $country,
				  'reffer' => $refferer,
				  'flg' => $flg ,
				  'info' => print_r($_SERVER,true)
				)
			);
		}
		
		if(!$isOK){
		
			$opt1 = get_option('limitlogin_mailaddress');
			$send_address = $opt1;
			$opt2 = get_option('limitlogin_logdatecount');
			$logdatecount = isset($opt2) && $opt2!='' ? $opt2: 90;
	
			//通知
			if($send_address != ""){
				$host = $_SERVER['HTTP_HOST'];
				$message = <<< DOC
$host
管理画面への不正アクセスを検知しました。
$ip
$refferer
DOC;
				wp_mail($send_address, 'Wordpress 不正アクセス検知('.$host.')', $message);
			}
			
			//ログ削除処理
			$table_name = $wpdb->prefix.'limitlogin_log';
			$wpdb->query( 
				$wpdb->prepare("DELETE FROM $table_name WHERE time < %s",
					date('Y-m-d H:i:s',strtotime("-".$logdatecount." day"))
				)
			);
			
			//処理中断
			if(!headers_sent()){
				wp_redirect( '/', 301 );
			}
			exit();
		}
	}
}
$limitjalogin = new limitjalogin;